/**
 * {加载条}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/04/26 15:20:28
 */

import { html, css, Component, styleMap } from 'wkit'

class Loading extends Component {
  static props = {
    progress: 'num!0',
    duration: 3000,
    value: {
      type: Boolean,
      observer(v) {
        if (this.duration > 0) {
          if (v) {
            requestAnimationFrame(this.#loading.bind(this))
          }
        } else {
          if (v) {
            this.setAttribute('loading', '')
          } else {
            this.removeAttribute('loading')
          }
        }
      }
    }
  }

  static styles = [
    css`
      :host {
        position: fixed;
        top: 0;
        left: 0;
        z-index: 65535;
        display: flex;
        width: 100vw;
        height: 0;
      }

      .thumb {
        width: 0;
        height: var(--wc-loading-height, 2px);
        background: var(--wc-loading-background, var(--color-teal-1));
        box-shadow: 0 0 5px var(--wc-loading-background, var(--color-teal-1));
        will-change: width;
        transition: width 0.1s linear;
      }

      :host([loading]) {
        .thumb {
          width: 25% !important;
          animation: autoplay 1s infinite ease-in-out;
        }
      }

      @keyframes autoplay {
        from,
        to {
          transform: translateX(0);
        }
        50% {
          transform: translateX(calc(100vw - 100%));
        }
      }
    `
  ]

  #start

  #loading(stamp) {
    if (this.#start === void 0) {
      this.#start = stamp
    }

    let time = stamp - this.#start
    let tmp = ~~(time / (this.duration / 100))

    if (tmp !== this.progress) {
      this.progress = tmp
    }

    if (time > this.duration) {
      this.progress = 0
      this.#start = void 0
      this.value = false
      this.$emit('input')
      this.$emit('change')
      return
    }
    requestAnimationFrame(this.#loading.bind(this))
  }

  render() {
    let style = styleMap({ width: this.progress + '%' })
    return html` <span class="thumb" style=${style}></span> `
  }
}

Loading.reg('loading')
